"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const TestHandler_1 = require("./TestHandler");
const events_1 = require("events");
const matchers_1 = require("../conditions/matchers");
const jest_when_1 = require("jest-when");
const actions_1 = require("../conditions/actions");
const ordered_1 = require("./ordered");
const TwilioCall_1 = require("./TwilioCall");
class WsTestDouble extends events_1.EventEmitter {
    constructor(sendMock = jest.fn()) {
        super();
        this.sendMock = sendMock;
    }
    // eslint-disable-next-line no-dupe-class-members
    send(data, cbOrOptions, cb) {
        this.sendMock(data, cbOrOptions, cb);
    }
}
describe("Then response", () => {
    test("payload from Then directive sent to Twilio when condition is met", async () => {
        const transcriptionHandler = new events_1.EventEmitter();
        const testWithSingleCondition = {
            name: "",
            test: ordered_1.ordered([
                {
                    when: matchers_1.contains("Hello"),
                    then: actions_1.press("123"),
                },
            ]),
        };
        const dtmfBuffer = Buffer.from([0, 1, 2, 3]);
        const dtmfGenerator = {
            generate: jest.fn(),
        };
        jest_when_1.when(dtmfGenerator.generate).calledWith("123").mockReturnValue(dtmfBuffer);
        const testDouble = new WsTestDouble();
        const call = new TwilioCall_1.TwilioCall(testDouble, dtmfGenerator);
        new TestHandler_1.TestHandler(call, transcriptionHandler, testWithSingleCondition);
        testDouble.emit("message", JSON.stringify({ event: "start", streamSid: "test-stream-sid" }));
        await new Promise((resolve) => setTimeout(resolve, 200));
        transcriptionHandler.emit("transcription", {
            transcription: "Hello World",
        });
        expect(testDouble.sendMock).toHaveBeenCalledWith(JSON.stringify({
            event: "media",
            streamSid: "test-stream-sid",
            media: {
                payload: dtmfBuffer.toString("base64"),
            },
        }), undefined, undefined);
    });
});
describe("When conditions", () => {
    test("passed event emitted if no conditions", (done) => {
        const transcriptionHandler = new events_1.EventEmitter();
        const testWithEmptyConditions = {
            name: "",
            test: ordered_1.ordered([]),
        };
        const call = new TwilioCall_1.TwilioCall(new WsTestDouble(), {
            generate: jest.fn(),
        });
        const handler = new TestHandler_1.TestHandler(call, transcriptionHandler, testWithEmptyConditions);
        handler.on("TestPassed", () => done());
        transcriptionHandler.emit("transcription", {
            transcription: "Hello World",
        });
    });
    test("passed event emitted if all conditions pass", (done) => {
        const transcriptionHandler = new events_1.EventEmitter();
        const testWithTwoCondition = {
            name: "",
            test: ordered_1.ordered([
                {
                    when: matchers_1.contains("1"),
                    then: actions_1.doNothing(),
                },
                {
                    when: matchers_1.contains("2"),
                    then: actions_1.doNothing(),
                },
            ]),
        };
        const call = new TwilioCall_1.TwilioCall(new WsTestDouble(), {
            generate: jest.fn(),
        });
        const handler = new TestHandler_1.TestHandler(call, transcriptionHandler, testWithTwoCondition);
        handler.on("TestPassed", () => done());
        transcriptionHandler.emit("transcription", {
            transcription: "Test transcript 1",
        });
        transcriptionHandler.emit("transcription", {
            transcription: "Test transcript 2",
        });
    });
    test("failure event emitted if all but last condition fails", (done) => {
        const transcriptionHandler = new events_1.EventEmitter();
        const testWithTwoConditions = {
            name: "",
            test: ordered_1.ordered([
                {
                    when: matchers_1.contains("1"),
                    then: actions_1.doNothing(),
                },
                {
                    when: matchers_1.contains("3"),
                    then: actions_1.doNothing(),
                },
            ]),
        };
        const call = new TwilioCall_1.TwilioCall(new WsTestDouble(), {
            generate: jest.fn(),
        });
        const handler = new TestHandler_1.TestHandler(call, transcriptionHandler, testWithTwoConditions);
        handler.on("TestFailed", () => done());
        transcriptionHandler.emit("transcription", {
            transcription: "Test transcript 1",
        });
        transcriptionHandler.emit("transcription", {
            transcription: "Test transcript 2",
        });
    });
    test("failure event emitted if first condition fails", (done) => {
        const transcriptionHandler = new events_1.EventEmitter();
        const twoConditions = {
            name: "",
            test: ordered_1.ordered([
                {
                    when: matchers_1.contains("3"),
                    then: actions_1.doNothing(),
                },
                {
                    when: matchers_1.contains("2"),
                    then: actions_1.doNothing(),
                },
            ]),
        };
        const call = new TwilioCall_1.TwilioCall(new WsTestDouble(), {
            generate: jest.fn(),
        });
        const handler = new TestHandler_1.TestHandler(call, transcriptionHandler, twoConditions);
        handler.on("TestFailed", () => done());
        transcriptionHandler.emit("transcription", {
            transcription: "Test transcript 1",
        });
        transcriptionHandler.emit("transcription", {
            transcription: "Test transcript 2",
        });
    });
});
