"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.startServerListening = exports.formatServerUrl = void 0;
const ws_1 = require("ws");
const TranscriptionHandler_1 = require("./handlers/TranscriptionHandler");
const MediaStreamRecorder_1 = require("./handlers/MediaStreamRecorder");
const TestHandler_1 = require("./handlers/TestHandler");
const TwilioCall_1 = require("./handlers/TwilioCall");
const url_1 = require("url");
exports.formatServerUrl = (server) => {
    const address = server.wss.address();
    switch (address.family) {
        case "IPv4":
            return new url_1.URL(`http://${address.address}:${address.port}`);
        case "IPv6": // https://tools.ietf.org/html/rfc2732#section-2
            return new url_1.URL(`http://[${address.address}]:${address.port}`);
        default:
            throw new Error(`Unrecognised '${address.family}' address family`);
    }
};
const initialiseConnectionHandlers = (wss, ws, config, ivrTest, testEventEmitter) => {
    const call = new TwilioCall_1.TwilioCall(ws, config.dtmfGenerator);
    if (config.recording) {
        new MediaStreamRecorder_1.MediaStreamRecorder(ws, config.recording, ivrTest);
    }
    const transcriptionHandler = new TranscriptionHandler_1.TranscriptionHandler(ws, config.transcriber());
    new TestHandler_1.TestHandler(call, transcriptionHandler, ivrTest)
        .on("ConditionMet", (event) => {
        testEventEmitter.emit("ivrTestConditionMet", event);
    })
        .on("TestPassed", (event) => {
        testEventEmitter.emit("ivrTestPassed", event);
    })
        .on("TestFailed", (event) => {
        testEventEmitter.emit("ivrTestFailed", event);
    });
};
exports.startServerListening = (config, ivrTest, testEventEmitter) => {
    const testIterator = ivrTest.entries();
    const wss = new ws_1.Server({ port: config.localServerPort });
    wss.on("connection", (ws) => {
        testEventEmitter.emit("callConnected", undefined);
        const testEntry = testIterator.next();
        if (!testEntry.done) {
            const [index, test] = testEntry.value;
            testEventEmitter.emit("callAssignedTest", { test, index });
            initialiseConnectionHandlers(wss, ws, config, test, testEventEmitter);
        }
    });
    return new Promise((resolve, reject) => {
        const onError = (err) => reject(err);
        wss.on("error", onError);
        wss.on("listening", () => {
            wss.off("error", onError);
            resolve({ wss });
        });
    });
};
